Package: cheem
Title: Interactively Explore Local Explanations with the Radial Tour
Version: 0.4.2
Authors@R: c(
  person("Nicholas", "Spyrison", role = c("aut", "cre"),
    email = "spyrison@gmail.com",
    comment = c(ORCID = "https://orcid.org/0000-0002-8417-0212"))
  )
Description: Given a non-linear model, calculate the local explanation. 
    We purpose view the data space, explanation 
    space, and model residuals as ensemble graphic interactive on a shiny 
    application. After an observation of interest is identified, the normalized 
    variable importance of the local explanation is used as a 1D projection
    basis. The support of the local explanation is then explored by changing
    the basis with the use of the radial tour <doi:10.32614/RJ-2020-027>; 
    <doi:10.1080/10618600.1997.10474754>.
Depends: R (>= 3.5.0)
License: MIT + file LICENSE
URL: https://github.com/nspyrison/cheem/,
        https://nspyrison.github.io/cheem/
BugReports: https://github.com/nspyrison/cheem/issues
Imports: spinifex (>= 0.3.3), ggplot2, plotly, magrittr, shiny,
        shinythemes, shinycssloaders, DT, conflicted
Suggests: methods, tourr, matrixcalc, mvtnorm, gganimate, dplyr, tidyr,
        tictoc, beepr, knitr, testthat (>= 3.0.0), rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Language: en-US
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-09-17 21:41:49 UTC; spyri
Author: Nicholas Spyrison [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8417-0212>)
Maintainer: Nicholas Spyrison <spyrison@gmail.com>
Repository: CRAN
Date/Publication: 2025-09-17 22:10:08 UTC
Built: R 4.6.0; ; 2026-04-30 19:13:47 UTC; unix
