# Generated by roxygen2: do not edit by hand

export("%>%")
export(as_logical_index)
export(cheem_ls)
export(color_scale_of)
export(contains_nonnumeric)
export(global_view)
export(is_discrete)
export(is_diverging)
export(linear_tform)
export(logistic_tform)
export(problem_type)
export(proto_basis1d_distribution)
export(radial_cheem_tour)
export(run_app)
export(subset_cheem)
export(sug_basis)
export(sug_manip_var)
importFrom(magrittr,"%>%")
